---------------------------------------------------------------------------------------------------------------------------------------
-- VICTOR MANUEL MIÑAMBRES CHAMORRO.
---------------------------------------------------------------------------------------------------------------------------------------
-- TFG -------------------- GLOBAL SERVICE DESK ---------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- Se crea el esquema GSD (Global Service Desk.
-- Se revocan los permisos a public (por defecto) al crear el objeto en base de datos.
-- Al final del script se dan los permisos a los usuarios de pruebas.
-- No se definen valores por defectos, ya que estos han de estar definidos por el departamento de desarrollo encargado del aplicativo.
-- Se establecen como piloto 1000 registros al año para el cálculo de extents.
-- Se crea una clave primaria en cada tabla por el código identificador.
-- Se crean las foreing keys correctas que establecen la relación con las tablas padre.
---------------------------------------------------------------------------------------------------------------------------------------

-- CREACION DE LA BASE DE DATOS GSD

drop database GSD;
create database GSD in cycprod_dat1 with buffered log;


--- CREACIÓN DEL MODELO DE DATOS


-- drop table Centers;

CREATE TABLE GSD.Centers
(
  centerCod VARCHAR(50),
  centerCountry VARCHAR(50),
  centerAddress VARCHAR(50),
  centerTypology VARCHAR(50)
  
) extent size 271  next size 56  lock mode row;

revoke all on Centers from "public" as "informix";

create unique index ix_Centers on Centers (centerCod) using btree  in cycprod_idx1;

alter table Centers add constraint primary key (centerCod) constraint pk_Centers ;

---------------------------------------------------------------------------------------------------------------------------------------
 
-- drop table Services;

CREATE TABLE GSD.Services
(
  serviceCod VARCHAR(50),
  serviceDescription VARCHAR(50),
  serviceGroup VARCHAR(50),
  servicePriority VARCHAR(50),
  serviceTimeRes VARCHAR(50)
  
) extent size 327  next size 68  lock mode row;

revoke all on Services from "public" as "informix";

create unique index ix_Services on Services (serviceCod) using btree  in cycprod_idx1;

alter table Services add constraint primary key (serviceCod) constraint pk_Services ;
---------------------------------------------------------------------------------------------------------------------------------------
  
-- drop table Support;

CREATE TABLE GSD.Support
(
  supportCod VARCHAR(50),
  supportName VARCHAR(50),
  supportType VARCHAR(50),
  supportuserCod VARCHAR(50)
  
) extent size 271  next size 56  lock mode row;

revoke all on Support from "public" as "informix";

create unique index ix_Support on Support (supportCod) using btree  in cycprod_idx1;

alter table Support add constraint primary key (supportCod) constraint pk_Support ;
---------------------------------------------------------------------------------------------------------------------------------------

-- drop table Users;

CREATE TABLE GSD.Users
(
  userCod VARCHAR(50),
  userReg DATETIME YEAR TO FRACTION(3),
  userUnsubscribe DATETIME YEAR TO FRACTION(3),
  userType VARCHAR(50),
  userSubscribe DATETIME YEAR TO FRACTION(3),
  usercenterCod VARCHAR(50),
  usersupportCod VARCHAR(50)
  
) extent size 406  next size 84  lock mode row;

revoke all on Users from "public" as "informix";

create unique index ix_Users on Users (userCod) using btree  in cycprod_idx1;

alter table Users add constraint primary key (userCod) constraint pk_Users ;

ALTER TABLE Users ADD CONSTRAINT (FOREIGN KEY (usercenterCod) REFERENCES Centers (centerCod) CONSTRAINT fk_Users_Centers);
ALTER TABLE Users ADD CONSTRAINT (FOREIGN KEY (usersupportCod) REFERENCES Support (supportCod) CONSTRAINT fk_Users_Support);

---------------------------------------------------------------------------------------------------------------------------------------

-- drop table Tickets;

CREATE TABLE GSD.Tickets
(
  ticketNum VARCHAR(50),
  ticketType VARCHAR(50),
  ticketuserCod VARCHAR(50),
  ticketCreate DATETIME YEAR TO FRACTION(3),
  ticketPriority VARCHAR(50),
  ticketService VARCHAR(50),
  ticketChannel VARCHAR(50),
  ticketCycle VARCHAR(50),
  ticketDescription LVARCHAR(2000),
  ticketmessageCod VARCHAR(50),
  ticketDocument CLOB

) extent size 4371  next size 876  lock mode row;

revoke all on Tickets from "public" as "informix";

create unique index ix_Tickets on Tickets (ticketNum) using btree  in cycprod_idx1;

alter table Tickets add constraint primary key (ticketNum) constraint pk_Tickets ;

ALTER TABLE Tickets ADD CONSTRAINT (FOREIGN KEY (ticketService) REFERENCES Services (serviceCod) CONSTRAINT fk_Tickets_Services);
ALTER TABLE Tickets ADD CONSTRAINT (FOREIGN KEY (ticketuserCod) REFERENCES Users (userCod) CONSTRAINT fk_Tickets_Users);

---------------------------------------------------------------------------------------------------------------------------------------

-- drop table State;

CREATE TABLE GSD.State
(
  stateCodState serial,
  stateTicketNum VARCHAR(50),
  stateChangeDate DATETIME YEAR TO FRACTION(3),
  stateUserCod VARCHAR(50),
  stateNew VARCHAR(50)
  
) extent size 226  next size 48  lock mode row;

revoke all on State from "public" as "informix";

create unique index ix_State on State (stateCodState) using btree  in cycprod_idx1;

alter table State add constraint primary key (stateCodState) constraint pk_State ;

ALTER TABLE State ADD CONSTRAINT (FOREIGN KEY (stateTicketNum) REFERENCES Tickets (ticketNum) CONSTRAINT fk_State_Tickets);
---------------------------------------------------------------------------------------------------------------------------------------

 -- drop table Messages;

CREATE TABLE GSD.Messages
(
  messageCod VARCHAR(50),
  messageTime DATETIME YEAR TO FRACTION(3),
  messageReceiver VARCHAR(50),
  messageState VARCHAR(50),
  messageTicketNum VARCHAR(50)
  
) extent size 226  next size 48  lock mode row;

revoke all on Messages from "public" as "informix";

create unique index ix_Messages on Messages (messageCod) using btree  in cycprod_idx1;

alter table Messages add constraint primary key (messageCod) constraint pk_Messages ;

ALTER TABLE Messages ADD CONSTRAINT (FOREIGN KEY (messageTicketNum) REFERENCES Tickets (ticketNum) CONSTRAINT fk_Messages_Tickets);
---------------------------------------------------------------------------------------------------------------------------------------
--	Tabla de registro de los procedimientos ejecutados.
---------------------------------------------------------------------------------------------------------------------------------------

-- drop table Log_procedures;

CREATE TABLE GSD.Log_procedures
(
  procCod serial,
  procName VARCHAR(100),
  procParamIn LVARCHAR(2000),
  procParamOut LVARCHAR(2000),
  procReg DATETIME YEAR TO FRACTION(3)
  
) extent size 226  next size 48  lock mode row;

revoke all on Log_procedures from "public" as "informix";

create unique index ix_Log_procedures on Log_procedures (procCod) using btree  in cycprod_idx1;

alter table Log_procedures add constraint primary key (procCod) constraint pk_Log_procedures ;
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------

-- PERMISOS NECESARIOS DEL PROTOTIPO.

-- PERMISOS DE CONEXION A LA BASE DE DATOS GSD

GRANT CONNECT TO USU1_GSD;
GRANT CONNECT TO USU2_GSD;

-- PERMISOS DE SELECT,INSERT,DELETE,UPDATE SOBRE EL MODELO AL USUARIO USU1_GSD;

GRANT SELECT,INSERT,DELETE,UPDATE ON Tickets  TO USU1_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Users    TO USU1_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Centers  TO USU1_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Services TO USU1_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON State    TO USU1_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Messages TO USU1_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Support  TO USU1_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Log_procedures  TO USU1_GSD  AS GSD;



-- PERMISOS DE SELECT,INSERT,DELETE,UPDATE SOBRE EL MODELO AL USUARIO USU2_GSD;

GRANT SELECT,INSERT,DELETE,UPDATE ON Tickets  TO USU2_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Users    TO USU2_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Centers  TO USU2_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Services TO USU2_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON State    TO USU2_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Messages TO USU2_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Support  TO USU2_GSD  AS GSD;
GRANT SELECT,INSERT,DELETE,UPDATE ON Log_procedures  TO USU2_GSD  AS GSD;

---------------------------------------------------------------------------------------------------------------------------------------




























