---------------------------------------------------------------------------------------------------------------------------------------
-- VICTOR MANUEL MIÑAMBRES CHAMORRO.
---------------------------------------------------------------------------------------------------------------------------------------
-- TFG -------------------- GLOBAL SERVICE DESK ---------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los procedures para el modelo.
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- Dado un mes cualquiera, tiempo medio de resolución de incidencias por cada tipo de prioridad.
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: Todos los campos de la tabla State (MENOS EL CAMPO SERIAL AUTOGENERADO).
-- PARAMETROS DE SALIDA: NO TIENE (El código RSP (RESULTADO DE LA OPERACIÓN) se registra en la tabla Log_procedures).
---------------------------------------------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS "GSD".timeSolInc;
CREATE PROCEDURE "GSD".timeSolInc (acc_timeSolInc VARCHAR(50), stateTicketNum_1 VARCHAR(50),stateChangeDate_1 DATETIME YEAR TO FRACTION(3), stateUserCod_1 VARCHAR(50), stateNew_1 VARCHAR(50));
---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Consulta_Usuario.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'maxTicketInc_insert. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el código SQL y el Código ISAM, para completar la descripción del error.
-- Se controlan las excepciones por el código ISAM que es mucho más explicativo, el SQL ERROR es más genérico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam INT;
DEFINE error_txt VARCHAR(100);
DEFINE cod_timeSolInc_1, month_1, numInc_1 INTEGER;
DEFINE HORA_INICIO, HORA_FIN DATETIME YEAR TO FRACTION(3);
DEFINE HORA_MEDIA, HORA_NEW, ticketType_sta VARCHAR(50);
DEFINE RSP, priority_1, timeAcu_1 VARCHAR(50);
DEFINE numInc_0,HORA_DIA_NEW,HORA_HORA_NEW,HORA_MINUTE_NEW INTEGER;
DEFINE FINAL_NEW,FINAL_ACUMULADO INTEGER;


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para más detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	
END EXCEPTION WITH RESUME
	

LET RSP = 'OK';

---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecución que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;

LET cod_timeSolInc_1 = 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Con la función MONTH se obtiene el mes de un campo DATETIME.
---------------------------------------------------------------------------------------------------------------------------------------
	LET month_1 = MONTH(stateChangeDate_1);
	
	SELECT stateChangeDate
	INTO HORA_INICIO
	FROM State 
	WHERE stateTicketNum = stateTicketNum_1 and stateNew = 'Creado'; 
	
	SELECT stateChangeDate
	INTO HORA_FIN
	FROM State 
	WHERE stateTicketNum = stateTicketNum_1 and stateNew = 'Resuelto'; 
	
	SELECT ticketPriority
	INTO priority_1
	FROM Tickets 
	WHERE ticketNum = stateTicketNum_1; 
	
	SELECT ticketType
	INTO ticketType_sta
	FROM Tickets 
	WHERE ticketNum = stateTicketNum_1; 

	SELECT numInc
	INTO numInc_0
	FROM timeSolInc 
	WHERE priority = priority_1  and month_tsi = month_1; 

	SELECT timeAcu
	INTO timeAcu_1
	FROM timeSolInc 
	WHERE priority = priority_1  and month_tsi = month_1;

---------------------------------------------------------------------------------------------------------------------------------------
-- Se incrementa el número de incidencias (numInc_1) y se calcula la media de tiempo que se tarda en solucionar una incidencia:
-- LET HORA_MEDIA = FINAL_ACUMULADO / numInc_1;
---------------------------------------------------------------------------------------------------------------------------------------

	
LET numInc_1 = numInc_0 + 1;

LET HORA_NEW = (HORA_FIN - HORA_INICIO);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se separa el año y la hora de los acumulados.
-- Se convierte los años y horas a minutos para dar el resultado en minutos.
--LET FINAL_NEW = (HORA_DIA_NEW*1440) + (HORA_HORA_NEW*60) + (HORA_MINUTE_NEW);
--LET FINAL_ACUMULADO = FINAL_NEW + timeAcu_1;
---------------------------------------------------------------------------------------------------------------------------------------

LET HORA_DIA_NEW = LEFT(HORA_NEW,10);
LET HORA_HORA_NEW = SUBSTR(HORA_NEW,11,2);
LET HORA_MINUTE_NEW = SUBSTR(HORA_NEW,14,2);

LET FINAL_NEW = (HORA_DIA_NEW*1440) + (HORA_HORA_NEW*60) + (HORA_MINUTE_NEW);
LET FINAL_ACUMULADO = FINAL_NEW + timeAcu_1;

LET HORA_MEDIA = FINAL_ACUMULADO / numInc_1;

---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza la actualización con los parámetros enviados en la tabla de timeSolInc, solo si el ticket es una incidencia.
---------------------------------------------------------------------------------------------------------------------------------------

IF ticketType_sta = 'INC' THEN

	IF acc_timeSolInc = 'Baja' OR acc_timeSolInc = 'Actualizar_Del' THEN
	
		---------------------------------------------------------------------------------------------------------------------------------------
		-- Se Decrementa el contador de incidencias si es una llamada de estado 'Resuelto' borrado.
		---------------------------------------------------------------------------------------------------------------------------------------
		
	
		UPDATE timeSolInc 
		SET  numInc=numInc - 1
		WHERE priority = priority_1 and month_tsi = month_1;
		
		---------------------------------------------------------------------------------------------------------------------------------------
		-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.	
		---------------------------------------------------------------------------------------------------------------------------------------
	
	
		INSERT INTO Log_procedures values (0, 'timeSolInc', 'SE DECREMENTA LA INCIDENCIA',RSP,CURRENT);
		
	ELSE
	
	
		UPDATE timeSolInc 
		SET  timeAcu=FINAL_ACUMULADO,timeSol=HORA_MEDIA,numInc=numInc_1
		WHERE priority = priority_1 and month_tsi = month_1;
		
		---------------------------------------------------------------------------------------------------------------------------------------
		-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
		---------------------------------------------------------------------------------------------------------------------------------------
	
	
		INSERT INTO Log_procedures values (0, 'timeSolInc', cod_timeSolInc_1||'//'||month_1||'//'||FINAL_ACUMULADO||'//'||HORA_MEDIA||'//'||priority_1||'//'||numInc_1,RSP,CURRENT);
		
	
	END IF

END IF


--	TRACE 'maxTicketInc_insert Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
update statistics for procedure;
---------------------------------------------------------------------------------------------------------------------------------------


















