---------------------------------------------------------------------------------------------------------------------------------------
-- VICTOR MANUEL MIÑAMBRES CHAMORRO.
---------------------------------------------------------------------------------------------------------------------------------------
-- TFG -------------------- GLOBAL SERVICE DESK ---------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los procedures para el modelo.
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- CONSULTA DE TICKETS EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: Número de Ticket(NUMERO_TICKET)
-- PARAMETROS DE SALIDA: Todos los campos de la tabla State.
---------------------------------------------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS "GSD".Consulta_Estado_Ticket;
CREATE PROCEDURE "GSD".Consulta_Estado_Ticket (ticketNum_1 VARCHAR(50)) RETURNING VARCHAR (100) AS RSP, INTEGER AS CODIGO_ESTADO, VARCHAR(50) AS CODIGO_TICKET, DATETIME YEAR TO FRACTION(3) AS FECHA_CAMBIO_ESTADO,
VARCHAR(50) AS CODIGO_USUARIO, VARCHAR(50) AS NUEVO_ESTADO;

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Consulta_CODIGO_TICKET.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Consulta_CODIGO_TICKET. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el código SQL y el Código ISAM, para completar la descripción del error.
-- Se controlan las excepciones por el código ISAM que es mucho más explicativo, el SQL ERROR es más genérico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador INT;
DEFINE error_txt VARCHAR(100);
DEFINE CODIGO_TICKET, CODIGO_USUARIO, NUEVO_ESTADO VARCHAR(50);
DEFINE FECHA_CAMBIO_ESTADO DATETIME YEAR TO FRACTION(3);
DEFINE CODIGO_ESTADO INTEGER; 
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para más detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecución que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza la consulta en la tabla de Tickets siempre que exista el Ticket, de no ser así se devuelve un mensaje comunicándolo.
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

LET CODIGO_ESTADO = NULL; 
LET CODIGO_TICKET = ''; 
LET FECHA_CAMBIO_ESTADO = ''; 
LET CODIGO_USUARIO = '';
LET NUEVO_ESTADO = ''; 

SELECT COUNT(*) 
INTO contador
from State where stateTicketNum = ticketNum_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL Ticket ' || ticketNum_1 || ' en la tabla State';
	
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
	---------------------------------------------------------------------------------------------------------------------------------------

	INSERT INTO Log_procedures values (0, 'Consulta_CODIGO_TICKET', ticketNum_1,RSP,CURRENT);

	RETURN 	RSP, CODIGO_ESTADO, CODIGO_TICKET , FECHA_CAMBIO_ESTADO, CODIGO_USUARIO, NUEVO_ESTADO;
	
ELSE

	LET RSP = 'OK';

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
	---------------------------------------------------------------------------------------------------------------------------------------

	INSERT INTO Log_procedures values (0, 'Consulta_CODIGO_TICKET', ticketNum_1,RSP,CURRENT);
	

	
	FOREACH SELECT *
	INTO CODIGO_ESTADO, CODIGO_TICKET , FECHA_CAMBIO_ESTADO, CODIGO_USUARIO, NUEVO_ESTADO
	FROM State 
	WHERE stateTicketNum = ticketNum_1
	
	RETURN 	RSP, CODIGO_ESTADO, CODIGO_TICKET , FECHA_CAMBIO_ESTADO, CODIGO_USUARIO, NUEVO_ESTADO WITH RESUME;
	
	END FOREACH;
	

	
END IF




	
--	TRACE 'Consulta_CODIGO_TICKET Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;

---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- ALTA DE ESTADOR DE TicketS EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: Todos los campos de la tabla State.
-- PARAMETROS DE SALIDA: RSP (RESULTADO DE LA OPERACIÓN).
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS "GSD".Alta_Estado_Ticket;
CREATE PROCEDURE "GSD".Alta_Estado_Ticket (stateTicketNum VARCHAR(50),stateChangeDate DATETIME YEAR TO FRACTION(3), stateUserCod VARCHAR(50), stateNew VARCHAR(50)) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

---		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Alta_Estado_Ticket.log";
---		TRACE ON; 
---		TRACE '-------------------------------------------------------------------';
---		TRACE 'Alta_Estado_Ticket. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el código SQL y el Código ISAM, para completar la descripción del error.
-- Se controlan las excepciones por el código ISAM que es mucho más explicativo, el SQL ERROR es más genérico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);
DEFINE CODIGO_ESTADO INTEGER;
DEFINE MONTH_1 INTEGER;
DEFINE HORA_MEDIA,HORA_INICIO,HORA_FIN VARCHAR(50);

ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam = 0 THEN

	LET RSP = 'OK'; 
	
ELSE

	IF error_isam = -111 THEN

		LET RSP = 'error -111 El ticket asociado al estado no existe'; 

	ELSE


		LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para más detalle compruebe el codigo ISAM mostrado ';
		
	END IF;


END IF;
	
END EXCEPTION WITH RESUME


LET RSP = 'OK';

---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecución que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;

LET CODIGO_ESTADO = 0; 

INSERT INTO State values (CODIGO_ESTADO , stateTicketNum , stateChangeDate , stateUserCod , stateNew );

---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Alta_Estado_Ticket', stateTicketNum||'//'||stateChangeDate||'//'||stateUserCod||'//'||stateNew,RSP,CURRENT);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se llama al procedimiento estadístico timeSolInc si el estado es resuelto:
---------------------------------------------------------------------------------------------------------------------------------------

IF stateNew = 'Resuelto' THEN

	CALL timeSolInc ('Alta',stateTicketNum ,stateChangeDate , stateUserCod , stateNew);

END IF


RETURN 	RSP;	

	
--	TRACE 'Alta_Estado_Ticket Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;

---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- BAJA DE ESTADOS  EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: CODIGO DEL TICKET Y ESTADO A BORRAR.
-- PARAMETROS DE SALIDA: RSP (RESULTADO DE LA OPERACIÓN).
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS "GSD".Baja_Estado_Ticket;
CREATE PROCEDURE "GSD".Baja_Estado_Ticket (stateTicketNum_1 VARCHAR(50), stateNew_1 VARCHAR(50)) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Baja_Estado_Ticket.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Baja_Estado_Ticket. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el código SQL y el Código ISAM, para completar la descripción del error.
-- Se controlan las excepciones por el código ISAM que es mucho más explicativo, el SQL ERROR es más genérico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);
DEFINE stateChangeDate_1 DATETIME YEAR TO FRACTION(3);
DEFINE stateUserCod_1 VARCHAR(50);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para más detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecución que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza el borrado en la tabla de usuarios siempre que exista el usuario, de no ser así se devuelve un mensaje comunicándolo.º
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

SELECT COUNT(*) 
INTO contador
from State where stateTicketNum = stateTicketNum_1 and stateNew = stateNew_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL ESTADO DEL TICKET ' || stateTicketNum_1 || ' en la tabla State';

ELSE
	
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se llama al procedimiento estadístico timeSolInc si el estado es resuelto:
	---------------------------------------------------------------------------------------------------------------------------------------
	
	IF stateNew_1 = 'Resuelto' THEN
	
	
		SELECT statechangedate 
		INTO stateChangeDate_1
		from State 
		where stateTicketNum = stateTicketNum_1 and stateNew = stateNew_1;
		
		SELECT stateusercod 
		INTO stateUserCod_1
		from State 
		where stateTicketNum = stateTicketNum_1 and stateNew = stateNew_1;
		
		CALL timeSolInc ('Baja',stateTicketNum_1 ,stateChangeDate_1 , stateUserCod_1 , stateNew_1);
			
	END IF
	
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se realiza el borrado en la tabla state:
	---------------------------------------------------------------------------------------------------------------------------------------
	
	delete from State where stateTicketNum = stateTicketNum_1 and stateNew = stateNew_1;
	LET RSP = 'OK';


	
	
		
		
END IF


---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Baja_Estado_Ticket', stateTicketNum_1||'//'||stateNew_1,RSP,CURRENT);



RETURN 	RSP;	

	
--	TRACE 'Baja_Estado_Ticket Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;

---------------------------------------------------------------------------------------------------------------------------------------
-- ACTUALIZAR ESTADOS DE UN TICKET EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: Todos los campos de la tabla State (MENOS EL CAMPO SERIAL AUTOGENERADO).
-- PARAMETROS DE SALIDA: RSP (RESULTADO DE LA OPERACIÓN).
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS "GSD".Actualiza_Estado_Tickets;
CREATE PROCEDURE "GSD".Actualiza_Estado_Tickets (statecodstate_1 INTEGER, stateTicketNum_1 VARCHAR(50),stateChangeDate_1 DATETIME YEAR TO FRACTION(3), stateUserCod_1 VARCHAR(50), stateNew_1 VARCHAR(50)) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Actualiza_Estado_Ticket.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Actualiza_Estado_Tickets. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el código SQL y el Código ISAM, para completar la descripción del error.
-- Se controlan las excepciones por el código ISAM que es mucho más explicativo, el SQL ERROR es más genérico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador  INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);
DEFINE COMPROBAR_ESTADO VARCHAR(50);


ON EXCEPTION SET error_num, error_isam, error_txt


IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para más detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	

	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecución que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza las comprobaciones de la existencia del usuario, del código de soporte y de Centro.
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

SELECT COUNT(*) 
INTO contador
from State where stateTicketNum = stateTicketNum_1 and statecodstate = statecodstate_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL ESTADO DEL TICKET ' || stateTicketNum_1 || ' en la tabla State';

ELSE

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se guarda el estado de la tabla State para comprobaciones estadísticas.
	---------------------------------------------------------------------------------------------------------------------------------------

	SELECT statenew 
	INTO COMPROBAR_ESTADO
	from State 
	where stateTicketNum = stateTicketNum_1 and statecodstate = statecodstate_1;

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se Actualiza la tabla State con los datos facilitados.
	---------------------------------------------------------------------------------------------------------------------------------------
	update State 
	set stateTicketNum=stateTicketNum_1,stateChangeDate=stateChangeDate_1,stateUserCod=stateUserCod_1,stateNew=stateNew_1
	where stateTicketNum = stateTicketNum_1 and statecodstate = statecodstate_1;
	
	LET RSP = 'OK';
			
			
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se llama al procedimiento estadístico timeSolInc si el estado es resuelto o si se modifica el estado del Resuelto a otro:
	-- Si se quita un estado 'Resuelto' hay que borrarlo de la tabla de estadística.
	---------------------------------------------------------------------------------------------------------------------------------------
	
	IF COMPROBAR_ESTADO = 'Resuelto' AND stateNew_1 <> 'Resuelto' THEN
	
	
		CALL timeSolInc ('Actualizar_Del',stateTicketNum_1 ,stateChangeDate_1 , stateUserCod_1 , stateNew_1);
		
				
	ELSE
	
		IF COMPROBAR_ESTADO = 'Resuelto' AND stateNew_1 = 'Resuelto' THEN 
	
		---------------------------------------------------------------------------------------------------------------------------------------
		-- NO hacemos nada, ya que no se modifica el estado de la tabla, se pueden modificar otros parámetros.
		---------------------------------------------------------------------------------------------------------------------------------------
		
		ELSE
		
			CALL timeSolInc ('Actualizar_Add',stateTicketNum_1 ,stateChangeDate_1 , stateUserCod_1 , stateNew_1);
		
		END IF
		
		
	
	END IF
	
	
END IF


---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Actualiza_Estado_Tickets', statecodstate_1||'//'||stateTicketNum_1||'//'||stateChangeDate_1||'//'||stateUserCod_1||'//'||stateNew_1,RSP,CURRENT);



RETURN 	RSP;	

	
--	TRACE 'Actualiza_Estado_Tickets Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se ejecutan estadísitcas para sincronizar el catálogo del gestor con los datos en disco, actualizando la caché.
---------------------------------------------------------------------------------------------------------------------------------------
update statistics for procedure;


