---------------------------------------------------------------------------------------------------------------------------------------
-- VICTOR MANUEL MIÑAMBRES CHAMORRO.
---------------------------------------------------------------------------------------------------------------------------------------
-- TFG -------------------- GLOBAL SERVICE DESK ---------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- Se defienen los procedures para el modelo.
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- CONSULTA DE MENSAJES EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: CODIGO_TICKET
-- PARAMETROS DE SALIDA: Resultado de la operación y  todos los campos de la tabla Messages. 
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS "GSD".Consulta_Mensajes_Ticket;
CREATE PROCEDURE "GSD".Consulta_Mensajes_Ticket (messageTicketNum_1 VARCHAR(50)) RETURNING VARCHAR (100) AS RSP, VARCHAR(50) AS CODIGO_MENSAJE, DATETIME YEAR TO FRACTION(3) AS FECHA_MENSAJE
, VARCHAR(50) AS USUARIO_MENSAJE, VARCHAR(50) AS ESTADO_MENSAJE , VARCHAR(50) AS CODIGO_TICKET;

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Consulta_Mensajes_Ticket.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Consulta_Mensajes_Ticket. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el código SQL y el Código ISAM, para completar la descripción del error.
-- Se controlan las excepciones por el código ISAM que es mucho más explicativo, el SQL ERROR es más genérico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador INT;
DEFINE error_txt VARCHAR(100);
DEFINE CODIGO_MENSAJE, USUARIO_MENSAJE, ESTADO_MENSAJE, CODIGO_TICKET VARCHAR(50);
DEFINE FECHA_MENSAJE DATETIME YEAR TO FRACTION(3);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para más detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecución que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza la consulta en la tabla de usuarios siempre que exista el usuario, de no ser así se devuelve un mensaje comunicándolo.
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

LET CODIGO_MENSAJE = '';
LET FECHA_MENSAJE = '';
LET USUARIO_MENSAJE = '';
LET ESTADO_MENSAJE = '';
LET CODIGO_TICKET = '';

SELECT COUNT(*) 
INTO contador
from Messages where messageTicketNum = messageTicketNum_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE MENSAJES PARA  ' || messageTicketNum_1 || ' en la tabla Messages';
	
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
	---------------------------------------------------------------------------------------------------------------------------------------
	
	INSERT INTO Log_procedures values (0, 'Consulta_Mensajes_Ticket', messageTicketNum_1,RSP,CURRENT);
	
	RETURN 	RSP, CODIGO_MENSAJE , FECHA_MENSAJE,  USUARIO_MENSAJE , ESTADO_MENSAJE, CODIGO_TICKET ;

ELSE
	
	LET RSP = 'OK';
	
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
	---------------------------------------------------------------------------------------------------------------------------------------
	
	INSERT INTO Log_procedures values (0, 'Consulta_Mensajes_Ticket', messageTicketNum_1,RSP,CURRENT);
	
	FOREACH SELECT * 
	INTO CODIGO_MENSAJE , FECHA_MENSAJE,  USUARIO_MENSAJE , ESTADO_MENSAJE, CODIGO_TICKET 
	FROM Messages 
	WHERE messageTicketNum = messageTicketNum_1
	
	RETURN 	RSP, CODIGO_MENSAJE , FECHA_MENSAJE,  USUARIO_MENSAJE , ESTADO_MENSAJE, CODIGO_TICKET WITH RESUME;
	
	END FOREACH;
	

	
END IF


	
--	TRACE 'Consulta_Mensajes_Ticket Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- ALTA DE MENSAJES EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: Todos los campos de la tabla Messages.
-- PARAMETROS DE SALIDA: RSP (RESULTADO DE LA OPERACIÓN).
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS "GSD".Alta_Mensajes_Ticket;
CREATE PROCEDURE "GSD".Alta_Mensajes_Ticket (messageCod VARCHAR(50), messageTime DATETIME YEAR TO FRACTION(3), messageReceiver VARCHAR(50),
  messageState VARCHAR(50), messageTicketNum VARCHAR(50)) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Alta_Mensajes_Ticket.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Alta_Mensajes_Ticket. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el código SQL y el Código ISAM, para completar la descripción del error.
-- Se controlan las excepciones por el código ISAM que es mucho más explicativo, el SQL ERROR es más genérico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam = 0 THEN

	LET RSP = 'OK'; 
	
ELSE

	IF error_isam = -111 THEN

		LET RSP = 'error -111 El Ticket al que asociar el mensaje no existe'; 

	ELSE


		LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para mas detalle compruebe el codigo ISAM mostrado ';
		
	END IF;


END IF;
	
END EXCEPTION WITH RESUME

LET RSP = 'OK';

---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecución que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza la inserción con los parámetros enviados en la tabla de Usuarios.
---------------------------------------------------------------------------------------------------------------------------------------
	
INSERT INTO Messages values (messageCod , messageTime , messageReceiver , messageState , messageTicketNum);


---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Alta_Mensajes_Ticket', messageCod||'//'||messageTime||'//'||messageReceiver||'//'||messageState||'//'||messageTicketNum,RSP,CURRENT);



RETURN 	RSP;	

	
--	TRACE 'Alta_Mensajes_Ticket Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- BAJA DE MENSAJES  EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: CODIGO DEL TICKET Y CODIGO DEL MENSAJE.
-- PARAMETROS DE SALIDA: RSP (RESULTADO DE LA OPERACIÓN).
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS "GSD".Baja_Estado_Ticket;
CREATE PROCEDURE "GSD".Baja_Estado_Ticket (messageTicketNum_1 VARCHAR(50), messageCod_1 VARCHAR(50)) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Baja_Estado_Ticket.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Baja_Estado_Ticket. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el código SQL y el Código ISAM, para completar la descripción del error.
-- Se controlan las excepciones por el código ISAM que es mucho más explicativo, el SQL ERROR es más genérico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para más detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecución que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza el borrado en la tabla de usuarios siempre que exista el usuario, de no ser así se devuelve un mensaje comunicándolo.º
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

SELECT COUNT(*) 
INTO contador
from Messages where messageTicketNum = messageTicketNum_1 and messageCod = messageCod_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL ESTADO DEL TICKET ' || messageTicketNum_1 || ' en la tabla Message';

ELSE
	
	delete from Messages where messageTicketNum = messageTicketNum_1 and messageCod = messageCod_1;
	LET RSP = 'OK';
	
END IF


---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Baja_Mensaje_Ticket', messageTicketNum_1||'//'||messageCod_1,RSP,CURRENT);



RETURN 	RSP;	

	
--	TRACE 'Baja_Estado_Ticket Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;

---------------------------------------------------------------------------------------------------------------------------------------
-- ACTUALIZAR MENSAJES DE UN TICKET EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: Todos los campos de la tabla Message.
-- PARAMETROS DE SALIDA: RSP (RESULTADO DE LA OPERACIÓN).
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS "GSD".Actualiza_Mensajes_Tickets;
CREATE PROCEDURE "GSD".Actualiza_Mensajes_Tickets (messageCod_1 VARCHAR(50), messageTime_1 DATETIME YEAR TO FRACTION(3), messageReceiver_1 VARCHAR(50),
  messageState_1 VARCHAR(50), messageTicketNum_1 VARCHAR(50)) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Actualiza_Estado_Ticket.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Actualiza_Mensajes_Tickets. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el código SQL y el Código ISAM, para completar la descripción del error.
-- Se controlan las excepciones por el código ISAM que es mucho más explicativo, el SQL ERROR es más genérico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador  INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt


IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para más detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	

	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecución que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza las comprobaciones de la existencia del usuario, del código de soporte y de Centro.
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

SELECT COUNT(*) 
INTO contador
from Messages where messageTicketNum = messageTicketNum_1 and messageCod = messageCod_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL Mensaje DEL TICKET ' || messageTicketNum_1 || ' en la tabla Messages';

ELSE

			update Messages 
			set messageCod=messageCod_1,messageTime=messageTime_1,messageReceiver=messageReceiver_1,messageState=messageState_1,messageTicketNum=messageTicketNum_1
			where messageTicketNum = messageTicketNum_1 and messageCod = messageCod_1;
			
			LET RSP = 'OK';
	
END IF


---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parámetros de entrada y de salida, hora de la ejecución.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Actualiza_Mensajes_Tickets', messageCod_1||'//'||messageTime_1||'//'||messageReceiver_1||'//'||messageState_1||'//'||messageTicketNum_1,RSP,CURRENT);




RETURN 	RSP;	

	
--	TRACE 'Actualiza_Mensajes_Tickets Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;


---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- Se ejecutan estadísitcas para sincronizar el catálogo del gestor con los datos en disco, actualizando la caché.
---------------------------------------------------------------------------------------------------------------------------------------
update statistics for procedure;
