---------------------------------------------------------------------------------------------------------------------------------------
-- VICTOR MANUEL MIAMBRES CHAMORRO.
---------------------------------------------------------------------------------------------------------------------------------------
-- TFG -------------------- GLOBAL SERVICE DESK ---------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los procedures para el modelo.
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- CONSULTA DE TICKETS EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: Nmero de Ticket(NUMERO_TICKET)
-- PARAMETROS DE SALIDA: Todos los campos de la tabla Ticket.
---------------------------------------------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS "GSD".Consulta_Ticket;
CREATE PROCEDURE "GSD".Consulta_Ticket (ticketNum_1 VARCHAR(50)) RETURNING VARCHAR (100) AS RSP, VARCHAR(50) AS NUMERO_TICKET, VARCHAR(50) AS TIPO_TICKET, VARCHAR(50) AS CODIGO_USUARIO, 
  DATETIME YEAR TO FRACTION(3) AS FECHA_CREACION_TICKET, VARCHAR(50) AS PRIORIDAD_TICKET, VARCHAR(50) AS SERVICIO_TICKET, VARCHAR(50) AS CANAL_TICKET,
  VARCHAR(50) AS CICLO_TICKET, LVARCHAR(2000) AS DESCRIPCION_TICKET, VARCHAR(50) AS CODIGO_MENSAJE, CLOB AS DOCUMENTO_ASOCIADO;

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Consulta_Ticket.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Consulta_Ticket. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el cdigo SQL y el Cdigo ISAM, para completar la descripcin del error.
-- Se controlan las excepciones por el cdigo ISAM que es mucho ms explicativo, el SQL ERROR es ms genrico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador INT;
DEFINE error_txt VARCHAR(100);
DEFINE NUMERO_TICKET, TIPO_TICKET, CODIGO_USUARIO, PRIORIDAD_TICKET, SERVICIO_TICKET, CANAL_TICKET, CICLO_TICKET, CODIGO_MENSAJE VARCHAR(50);
DEFINE FECHA_CREACION_TICKET DATETIME YEAR TO FRACTION(3);
DEFINE DESCRIPCION_TICKET LVARCHAR(2000);
DEFINE DOCUMENTO_ASOCIADO CLOB;
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para ms detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecucin que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza la consulta en la tabla de Tickets siempre que exista el Ticket, de no ser as se devuelve un mensaje comunicndolo.
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

LET NUMERO_TICKET = ''; 
LET TIPO_TICKET  = '';
LET CODIGO_USUARIO  = ''; 
LET FECHA_CREACION_TICKET  = ''; 
LET PRIORIDAD_TICKET  = '';
LET SERVICIO_TICKET  = ''; 
LET CANAL_TICKET  = ''; 
LET CICLO_TICKET  = ''; 
LET DESCRIPCION_TICKET  = ''; 
LET CODIGO_MENSAJE  = ''; 
LET DOCUMENTO_ASOCIADO  = NULL;



SELECT COUNT(*) 
INTO contador
from Tickets where ticketNum = ticketNum_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL Ticket ' || ticketNum_1 || ' en la tabla Tickets';

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parmetros de entrada y de salida, hora de la ejecucin.
	---------------------------------------------------------------------------------------------------------------------------------------

	INSERT INTO Log_procedures values (0, 'Consulta_Ticket', ticketNum_1,RSP,CURRENT);
	
	RETURN 	RSP, NUMERO_TICKET, TIPO_TICKET , CODIGO_USUARIO , FECHA_CREACION_TICKET , PRIORIDAD_TICKET ,  SERVICIO_TICKET ,  CANAL_TICKET , CICLO_TICKET, DESCRIPCION_TICKET , CODIGO_MENSAJE , DOCUMENTO_ASOCIADO;	

	
ELSE
	
	LET RSP = 'OK';
	
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parmetros de entrada y de salida, hora de la ejecucin.
	---------------------------------------------------------------------------------------------------------------------------------------

	INSERT INTO Log_procedures values (0, 'Consulta_Ticket', ticketNum_1,RSP,CURRENT);
	
	FOREACH SELECT *
	INTO NUMERO_TICKET, TIPO_TICKET , CODIGO_USUARIO , FECHA_CREACION_TICKET , PRIORIDAD_TICKET ,  SERVICIO_TICKET ,  CANAL_TICKET , CICLO_TICKET, DESCRIPCION_TICKET , CODIGO_MENSAJE , DOCUMENTO_ASOCIADO
	FROM Tickets 
	WHERE ticketNum = ticketNum_1
	
	RETURN 	RSP, NUMERO_TICKET, TIPO_TICKET , CODIGO_USUARIO , FECHA_CREACION_TICKET , PRIORIDAD_TICKET ,  SERVICIO_TICKET ,  CANAL_TICKET , CICLO_TICKET, DESCRIPCION_TICKET , CODIGO_MENSAJE , DOCUMENTO_ASOCIADO WITH RESUME;
	
	END FOREACH;
	
	
	
END IF

--	TRACE 'Consulta_Ticket Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;

---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- ALTA DE TicketS EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: Todos los campos de la tabla Tickets.
-- PARAMETROS DE SALIDA: RSP (RESULTADO DE LA OPERACIN).
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS "GSD".Alta_Ticket;
CREATE PROCEDURE "GSD".Alta_Ticket (ticketNum VARCHAR(50),
  ticketType VARCHAR(50),
  ticketuserCod VARCHAR(50),
  ticketCreate DATETIME YEAR TO FRACTION(3),
  ticketPriority VARCHAR(50),
  ticketService VARCHAR(50),
  ticketChannel VARCHAR(50),
  ticketCycle VARCHAR(50),
  ticketDescription LVARCHAR(2000),
  ticketmessageCod VARCHAR(50),
  ticketDocument CLOB) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Consulta_Ticket.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Alta_Ticket. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el cdigo SQL y el Cdigo ISAM, para completar la descripcin del error.
-- Se controlan las excepciones por el cdigo ISAM que es mucho ms explicativo, el SQL ERROR es ms genrico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam = 0 THEN

	LET RSP = 'OK'; 
	
ELSE

	IF error_isam = -100 THEN

		LET RSP = 'error -100 El Ticket ya existe'; 

	ELSE


		LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para ms detalle compruebe el codigo ISAM mostrado ';
		
	END IF;


END IF;
	
END EXCEPTION WITH RESUME

LET RSP = 'OK';


---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecucin que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza la insercin con los parmetros enviados en la tabla de Tickets.
-- Para las inserciones del campo CLOB (ticketDocument) se tiene que utilizar la funcin:
-- FILETOCLOB('/backups_bbdd/tmp/GSD/PruebaClob.txt', 'server') o utilizar 'client' si el archivo reside en el cliente.
-- Al ser un punto a gestionar por el aplicativo para el prototipo se utiliza NULL.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Tickets values (ticketNum, ticketType, ticketuserCod, ticketCreate, ticketPriority, ticketService, ticketChannel, ticketCycle, ticketDescription, ticketmessageCod, ticketDocument);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parmetros de entrada y de salida, hora de la ejecucin.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Alta_Ticket', ticketNum||'//'||ticketType||'//'||ticketuserCod||'//'||ticketCreate||'//'||ticketPriority||'//'||ticketService||'//'||ticketChannel||'//'||ticketCycle||'//'||ticketDescription||'//'||ticketmessageCod,RSP,CURRENT);



RETURN 	RSP;	

	
--	TRACE 'Alta_Ticket Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- BAJA DE TicketS EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS "GSD".Baja_Ticket;
CREATE PROCEDURE "GSD".Baja_Ticket (ticketNum_1 VARCHAR(50)) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Baja_Ticket.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Baja_Ticket. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el cdigo SQL y el Cdigo ISAM, para completar la descripcin del error.
-- Se controlan las excepciones por el cdigo ISAM que es mucho ms explicativo, el SQL ERROR es ms genrico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para ms detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecucin que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza el borrado en la tabla de Tickets siempre que exista el Ticket, de no ser as se devuelve un mensaje comunicndolo.
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

SELECT COUNT(*) 
INTO contador
from Tickets where ticketNum = ticketNum_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL Ticket ' || ticketNum_1 || ' en la tabla Tickets';

ELSE
	
	delete from Tickets where ticketNum = ticketNum_1;
	LET RSP = 'OK';
	
END IF


---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parmetros de entrada y de salida, hora de la ejecucin.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Baja_Ticket', ticketNum_1,RSP,CURRENT);



RETURN 	RSP;	

	
--	TRACE 'Baja_Ticket Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;

---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- ACTUALIZAR TicketS EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS "GSD".Actualiza_Ticket;
CREATE PROCEDURE "GSD".Actualiza_Ticket (ticketNum_1 VARCHAR(50),
  ticketType_1 VARCHAR(50),
  ticketuserCod_1 VARCHAR(50),
  ticketCreate_1 DATETIME YEAR TO FRACTION(3),
  ticketPriority_1 VARCHAR(50),
  ticketService_1 VARCHAR(50),
  ticketChannel_1 VARCHAR(50),
  ticketCycle_1 VARCHAR(50),
  ticketDescription_1 LVARCHAR(2000),
  ticketmessageCod_1 VARCHAR(50),
  ticketDocument_1 CLOB) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Alta_Ticket.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Actualiza_Ticket. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el cdigo SQL y el Cdigo ISAM, para completar la descripcin del error.
-- Se controlan las excepciones por el cdigo ISAM que es mucho ms explicativo, el SQL ERROR es ms genrico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador, contador2, contador3 INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt


IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para ms detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	

	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecucin que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza las comprobaciones de la existencia del Ticket, del cdigo de soporte y de Centro.
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

SELECT COUNT(*) 
INTO contador
from Tickets where ticketNum = ticketNum_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL Ticket ' || ticketNum_1 || ' en la tabla Tickets';

ELSE

	SELECT COUNT(*) 
	INTO contador2
	from Services where serviceCod = ticketService_1;	

	IF contador2 = 0 THEN

	LET RSP = ' NO EXISTE EL CODIGO DE SERVICIO ' || ticketService_1 || ' en la tabla Services ';

	ELSE
	
		SELECT COUNT(*) 
		INTO contador3
		from Users where userCod = ticketuserCod_1;	

		IF contador3 = 0 THEN

		LET RSP = ' NO EXISTE EL CODIGO DE USUARIO ' || ticketuserCod_1 || ' en la tabla Users';
	
		ELSE
	
		---------------------------------------------------------------------------------------------------------------------------------------
		-- Se realiza la insercin con los parmetros enviados en la tabla de Tickets.
		-- Para las inserciones del campo CLOB (ticketDocument) se tiene que utilizar la funcin:
		-- FILETOCLOB('/backups_bbdd/tmp/GSD/PruebaClob.txt', 'server') o utilizar 'client' si el archivo reside en el cliente.
		-- Al ser un punto a gestionar por el aplicativo para el prototipo se utiliza NULL.
		---------------------------------------------------------------------------------------------------------------------------------------

			update Tickets 
			set ticketNum=ticketNum_1,ticketType=ticketType_1,ticketuserCod=ticketuserCod_1,ticketCreate=ticketCreate_1,ticketPriority=ticketPriority_1,ticketService=ticketService_1,ticketChannel=ticketChannel_1,ticketCycle=ticketCycle_1,ticketDescription=ticketDescription_1,ticketmessageCod=ticketmessageCod_1,ticketDocument=ticketDocument_1
			where ticketNum = ticketNum_1;
			
			LET RSP = 'OK';
			
		END IF
	
	END IF
	
END IF


---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parmetros de entrada y de salida, hora de la ejecucin.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Actualiza_Ticket', ticketNum_1||'//'||ticketType_1||'//'||ticketuserCod_1||'//'||ticketCreate_1||'//'||ticketPriority_1||'//'||ticketService_1||'//'||ticketChannel_1||'//'||ticketCycle_1||'//'||ticketDescription_1||'//'||ticketmessageCod_1,RSP,CURRENT);




RETURN 	RSP;	

	
--	TRACE 'Actualiza_Ticket Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;

---------------------------------------------------------------------------------------------------------------------------------------
-- Se ejecutan estadsitcas para sincronizar el catlogo del gestor con los datos en disco, actualizando la cach.
---------------------------------------------------------------------------------------------------------------------------------------
update statistics for procedure;



