---------------------------------------------------------------------------------------------------------------------------------------
-- VICTOR MANUEL MIAMBRES CHAMORRO.
---------------------------------------------------------------------------------------------------------------------------------------
-- TFG -------------------- GLOBAL SERVICE DESK ---------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- Se defienen los procedures para el modelo.
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- CONSULTA DE USUARIOS EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: CODIGO_USUARIO
-- PARAMETROS DE SALIDA: Todos los campos de la tabla Users.
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS "GSD".Consulta_Usuario;
CREATE PROCEDURE "GSD".Consulta_Usuario (userCod_1 VARCHAR(50)) RETURNING VARCHAR (100) AS RSP, VARCHAR(50) AS CODIGO_USUARIO, DATETIME YEAR TO FRACTION(3) AS FECHA_REGISTRO_USUARIO
, DATETIME YEAR TO FRACTION(3) AS FECHA_BAJA_USUARIO, VARCHAR(50) AS TIPO_USUARIO, DATETIME YEAR TO FRACTION(3) AS FECHA_SUBSCRIPCION,  VARCHAR(50) AS CODIGO_CENTRO
, VARCHAR(50) AS CODIGO_SOPORTE;

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Consulta_Usuario.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Consulta_Usuario. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el cdigo SQL y el Cdigo ISAM, para completar la descripcin del error.
-- Se controlan las excepciones por el cdigo ISAM que es mucho ms explicativo, el SQL ERROR es ms genrico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador INT;
DEFINE error_txt VARCHAR(100);
DEFINE CODIGO_USUARIO, TIPO_USUARIO, CODIGO_CENTRO, CODIGO_SOPORTE VARCHAR(50);
DEFINE FECHA_REGISTRO_USUARIO, FECHA_BAJA_USUARIO, FECHA_SUBSCRIPCION DATETIME YEAR TO FRACTION(3);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para ms detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecucin que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza la consulta en la tabla de usuarios siempre que exista el usuario, de no ser as se devuelve un mensaje comunicndolo.
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

LET CODIGO_USUARIO = '';
LET FECHA_REGISTRO_USUARIO = '';
LET FECHA_BAJA_USUARIO = '';
LET TIPO_USUARIO = '';
LET FECHA_SUBSCRIPCION = '';
LET CODIGO_CENTRO = '';
LET CODIGO_SOPORTE = '';

SELECT COUNT(*) 
INTO contador
from Users where userCod = userCod_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL USUARIO ' || userCod_1 || ' en la tabla Users';
	
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parmetros de entrada y de salida, hora de la ejecucin.
	---------------------------------------------------------------------------------------------------------------------------------------
	
	INSERT INTO Log_procedures values (0, 'Consulta_Usuario', userCod_1,RSP,CURRENT);
	
	RETURN 	RSP, CODIGO_USUARIO, FECHA_REGISTRO_USUARIO , FECHA_BAJA_USUARIO , TIPO_USUARIO ,  FECHA_SUBSCRIPCION ,  CODIGO_CENTRO , CODIGO_SOPORTE;

ELSE
	
	LET RSP = 'OK';
	
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parmetros de entrada y de salida, hora de la ejecucin.
	---------------------------------------------------------------------------------------------------------------------------------------
	
	INSERT INTO Log_procedures values (0, 'Consulta_Usuario', userCod_1,RSP,CURRENT);
	
	FOREACH SELECT * 
	INTO CODIGO_USUARIO, FECHA_REGISTRO_USUARIO , FECHA_BAJA_USUARIO , TIPO_USUARIO ,  FECHA_SUBSCRIPCION ,  CODIGO_CENTRO , CODIGO_SOPORTE 
	FROM Users 
	WHERE userCod = userCod_1
	
	RETURN 	RSP, CODIGO_USUARIO, FECHA_REGISTRO_USUARIO , FECHA_BAJA_USUARIO , TIPO_USUARIO ,  FECHA_SUBSCRIPCION ,  CODIGO_CENTRO , CODIGO_SOPORTE WITH RESUME;
	
	END FOREACH;
	

	
END IF






	

	
--	TRACE 'Consulta_Usuario Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;

---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- ALTA DE USUARIOS EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: Todos los campos de la tabla Users.
-- PARAMETROS DE SALIDA: RSP (RESULTADO DE LA OPERACIN).
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS "GSD".Alta_Usuario;
CREATE PROCEDURE "GSD".Alta_Usuario (userCod VARCHAR(50), userReg DATETIME YEAR TO FRACTION(3), userUnsubscribe DATETIME YEAR TO FRACTION(3), userType VARCHAR(50),
  userSubscribe DATETIME YEAR TO FRACTION(3),  usercenterCod VARCHAR(50), usersupportCod VARCHAR(50)) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Consulta_Usuario.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Alta_Usuario. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el cdigo SQL y el Cdigo ISAM, para completar la descripcin del error.
-- Se controlan las excepciones por el cdigo ISAM que es mucho ms explicativo, el SQL ERROR es ms genrico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam = 0 THEN

	LET RSP = 'OK'; 
	
ELSE

	IF error_isam = -100 THEN

		LET RSP = 'error -100 El usuario ya existe'; 

	ELSE


		LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para ms detalle compruebe el codigo ISAM mostrado ';
		
	END IF;


END IF;
	
END EXCEPTION WITH RESUME

LET RSP = 'OK';

---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecucin que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza la insercin con los parmetros enviados en la tabla de Usuarios.
---------------------------------------------------------------------------------------------------------------------------------------
	
INSERT INTO Users values (userCod,userReg,userUnsubscribe,userType,userSubscribe,usercenterCod,usersupportCod);


---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parmetros de entrada y de salida, hora de la ejecucin.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Alta_Usuario', userCod||'//'||userReg||'//'||userUnsubscribe||'//'||userType||'//'||userSubscribe||'//'||usercenterCod||'//'||usersupportCod,RSP,CURRENT);



RETURN 	RSP;	

	
--	TRACE 'Alta_Usuario Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- BAJA DE USUARIOS EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: CODIGO_USUARIO
-- PARAMETROS DE SALIDA: CODIGO DE RETORNO RSP.
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS "GSD".Baja_Usuario;
CREATE PROCEDURE "GSD".Baja_Usuario (userCod_1 VARCHAR(50)) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Baja_Usuario.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Baja_Usuario. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el cdigo SQL y el Cdigo ISAM, para completar la descripcin del error.
-- Se controlan las excepciones por el cdigo ISAM que es mucho ms explicativo, el SQL ERROR es ms genrico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt

IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para ms detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecucin que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza el borrado en la tabla de usuarios siempre que exista el usuario, de no ser as se devuelve un mensaje comunicndolo.
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

SELECT COUNT(*) 
INTO contador
from Users where userCod = userCod_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL USUARIO ' || userCod_1 || ' en la tabla Users';

ELSE
	
	delete from Users where userCod = userCod_1;
	LET RSP = 'OK';
	
END IF


---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parmetros de entrada y de salida, hora de la ejecucin.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Baja_Usuario', userCod_1,RSP,CURRENT);



RETURN 	RSP;	

	
--	TRACE 'Baja_Usuario Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;

---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
-- ACTUALIZAR USUARIOS EN EL SISTEMA.
---------------------------------------------------------------------------------------------------------------------------------------
-- PARAMETROS DE ENTRADA: Todos los campos de la tabla Users.
-- PARAMETROS DE SALIDA: RSP (RESULTADO DE LA OPERACIN).
---------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS "GSD".Actualiza_Usuario;
CREATE PROCEDURE "GSD".Actualiza_Usuario (userCod_1 VARCHAR(50), userReg_1 DATETIME YEAR TO FRACTION(3), userUnsubscribe_1 DATETIME YEAR TO FRACTION(3), userType_1 VARCHAR(50),
  userSubscribe_1 DATETIME YEAR TO FRACTION(3),  usercenterCod_1 VARCHAR(50), usersupportCod_1 VARCHAR(50)) RETURNING VARCHAR (100);

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen las variables a utilizar:
-- RSP = CODIGO DE RETORNO OK Si se ha ejecutado correctamente el procedimiento.
---------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------
-- Se activa el debug a fichero del procedimiento por si hace falta para el control de errores del procedimiento:
---------------------------------------------------------------------------------------------------------------------------------------

--		SET DEBUG FILE TO "/backups_bbdd/tmp/GSD/Alta_Usuario.log";
--		TRACE ON; 
--		TRACE '-------------------------------------------------------------------';
--		TRACE 'Actualiza_Usuario. Inicio';  

---------------------------------------------------------------------------------------------------------------------------------------
-- Se definen los errores reportados por la base de datos:
-- Se devuelve el cdigo SQL y el Cdigo ISAM, para completar la descripcin del error.
-- Se controlan las excepciones por el cdigo ISAM que es mucho ms explicativo, el SQL ERROR es ms genrico.
---------------------------------------------------------------------------------------------------------------------------------------

DEFINE error_num, error_isam, contador, contador2, contador3 INT;
DEFINE error_txt VARCHAR(100);
DEFINE RSP VARCHAR(100);


ON EXCEPTION SET error_num, error_isam, error_txt


IF  error_isam <> 0 THEN

	LET RSP = 'Error ' || error_isam || ' en: ' || error_txt || ' , para ms detalle compruebe el codigo ISAM mostrado ';
		
END IF;
	

	
END EXCEPTION WITH RESUME



---------------------------------------------------------------------------------------------------------------------------------------
-- Se configura una prioridad a 0 por defecto, si es necesario se establece con el valor de prioridad de ejecucin que se necesite.
---------------------------------------------------------------------------------------------------------------------------------------
SET PDQPRIORITY 0;


---------------------------------------------------------------------------------------------------------------------------------------
-- Se realiza las comprobaciones de la existencia del usuario, del cdigo de soporte y de Centro.
---------------------------------------------------------------------------------------------------------------------------------------

LET contador = 0;

SELECT COUNT(*) 
INTO contador
from Users where userCod = userCod_1;

IF contador = 0 THEN

	LET RSP = ' NO EXISTE EL USUARIO ' || userCod_1 || ' en la tabla Users';

ELSE

	SELECT COUNT(*) 
	INTO contador2
	from Users where usercenterCod = usercenterCod_1;	

	IF contador2 = 0 THEN

	LET RSP = ' NO EXISTE EL CODIGO DE CENTRO ' || usercenterCod_1 || ' en la tabla Centers ';

	ELSE
	
		SELECT COUNT(*) 
		INTO contador3
		from Users where usersupportCod = usersupportCod_1;	

		IF contador3 = 0 THEN

		LET RSP = ' NO EXISTE EL CODIGO DE SOPORTE ' || usersupportCod_1 || ' en la tabla Support';
	
		ELSE
	
			update Users 
			set userCod=userCod_1,userReg=userReg_1,userUnsubscribe=userUnsubscribe_1,userType=userType_1,userSubscribe=userSubscribe_1,usercenterCod=usercenterCod_1,usersupportCod=usersupportCod_1 
			where userCod = userCod_1;
			
			LET RSP = 'OK';
			
		END IF
	
	END IF
	
END IF


---------------------------------------------------------------------------------------------------------------------------------------
-- Se inserta en la tabla de log de procedimientos, el nombre del procedimiento, parmetros de entrada y de salida, hora de la ejecucin.
---------------------------------------------------------------------------------------------------------------------------------------

INSERT INTO Log_procedures values (0, 'Actualiza_Usuario', userCod_1||'//'||userReg_1||'//'||userUnsubscribe_1||'//'||userType_1||'//'||userSubscribe_1||'//'||usercenterCod_1||'//'||usersupportCod_1,RSP,CURRENT);




RETURN 	RSP;	

	
--	TRACE 'Actualiza_Usuario Fin';
--		TRACE '-------------------------------------------------------------------';
--		TRACE OFF;
--	END IF;

END PROCEDURE;

---------------------------------------------------------------------------------------------------------------------------------------
-- Se ejecutan estadsitcas para sincronizar el catlogo del gestor con los datos en disco, actualizando la cach.
---------------------------------------------------------------------------------------------------------------------------------------
update statistics for procedure;



